///////////////////////////////////////////////////////////
// ODBUTFRM.CPP: Owner-Draw Button in FrameWindow
///////////////////////////////////////////////////////////
#include <owl\applicat.h>
#include <owl\framewin.h>
#include <stdio.h>
#include <string.h>
#include <owl\button.h>
#include <owl\control.h>
#include <owl\gdiobject.h>
#include <owl\edit.h>
#pragma hdrstop
#include "odbutfrm.rc"


class TODButton : public TButton {
public:
	TODButton(TWindow *parent, int resourceId, const char far* title,
		int x, int y, int w, int h, int resId1, int resId2, int resId3);
	~TODButton();

protected:
	virtual void ODADrawEntire(DRAWITEMSTRUCT far &drawInfo);
	virtual void ODASelect(DRAWITEMSTRUCT far &drawInfo);
	virtual void ODAFocus(DRAWITEMSTRUCT far &drawInfo);
	void DrawButton(DRAWITEMSTRUCT far &drawInfo, TBitmap *bm);

	TBitmap *bmNormal;
	TBitmap *bmPressed;
	TBitmap *bmFocused;
};

///////////////////////////////////////////////////////////
// TODButton::TODButton()
///////////////////////////////////////////////////////////
TODButton::TODButton(TWindow *parent, int resourceId, const char far* title,
	 int x, int y, int w, int h, int resId1, int resId2, int resId3)
	:	TButton(parent, resourceId, title, x, y, w, h)
{
	// Get a handle to the application's instance.
	HINSTANCE hInstance = *GetApplication();

	// Load the button bitmaps.
	bmNormal  = new TBitmap(hInstance, resId1);
	bmPressed = new TBitmap(hInstance, resId2);
	bmFocused = new TBitmap(hInstance, resId3);

	Attr.Style |= BS_OWNERDRAW;
}

///////////////////////////////////////////////////////////
// TODButton::~TODButton()
///////////////////////////////////////////////////////////
TODButton::~TODButton()
{
	// Delete the buttons.
	delete bmNormal;
	delete bmPressed;
	delete bmFocused;
}

///////////////////////////////////////////////////////////
// TODButton::ODADrawEntire()
///////////////////////////////////////////////////////////
void TODButton::ODADrawEntire(DRAWITEMSTRUCT far &drawInfo)
{
	// Draw the normal button face.
	DrawButton(drawInfo, bmNormal);
}

///////////////////////////////////////////////////////////
// TODButton::ODASelect()
///////////////////////////////////////////////////////////
void TODButton::ODASelect(DRAWITEMSTRUCT far &drawInfo)
{
	// If the button is selected...
	if (drawInfo.itemState & ODS_SELECTED)
		// ...draw the pressed button face.
		DrawButton(drawInfo, bmPressed);
	else
		// ...else draw the focused button face.
		DrawButton(drawInfo, bmFocused);
}

///////////////////////////////////////////////////////////
// TODButton::ODAFocus()
///////////////////////////////////////////////////////////
void TODButton::ODAFocus(DRAWITEMSTRUCT far &drawInfo)
{
	// If the button has the focus...
	if (drawInfo.itemState & ODS_FOCUS)
		// ...draw the focused button face.
		DrawButton(drawInfo, bmFocused);
	else
		// ...else draw the normal button face.
		DrawButton(drawInfo, bmNormal);
}

///////////////////////////////////////////////////////////
// TODButton::DrawButton()
///////////////////////////////////////////////////////////
void TODButton::DrawButton(DRAWITEMSTRUCT far &drawInfo, TBitmap *bm)
{
	// Get device contexts for the button and the bitmap.
	TDC dc(drawInfo.hDC);
	TMemoryDC memDC(dc);

	// Select the bitmap into the memory DC.
	memDC.SelectObject(*bm);

	// Calculate the button's size.
	int x = drawInfo.rcItem.left;
	int y = drawInfo.rcItem.top;
	int w = bm->Width();
	int h = bm->Height();

	// Draw the button.
	dc.BitBlt(x, y, w, h, memDC, 0, 0);
}


//////////////////////////////////
// The main window class.
//////////////////////////////////
class TWndw : public TFrameWindow
{
public:
	TWndw(TWindow *parent, const char far *title);
	~TWndw();
protected:
	void SetupWindow();
	void CleanupWindow();
	void CmOkButton1() {MessageBox("CM_OKBUTTON1 received\n"
		"Regular Button", "Message");}
	void CmOkButton2() {MessageBox("CM_OKBUTTON2 received\n"
		"Owner-Draw Button", "Message");}
private:
	TButton *button1;
	TODButton *button2;
	TEdit *edit;
DECLARE_RESPONSE_TABLE(TWndw);
};

DEFINE_RESPONSE_TABLE1(TWndw, TWindow)
	// Place response table entries here.
	EV_COMMAND(CM_OKBUTTON1, CmOkButton1),
	EV_COMMAND(CM_OKBUTTON2, CmOkButton2),
END_RESPONSE_TABLE;


///////////////////////////////////////////////////////////
// TWndw::TWndw()
///////////////////////////////////////////////////////////
TWndw::TWndw(TWindow *parent, const char far *title)
	:	TFrameWindow(parent, title)
{
	AssignMenu(MENU_1);
	// Perform object initialization here.

	Attr.X = 50;
	Attr.Y = 50;
	Attr.W = 400;
	Attr.H = 400;

	EnableKBHandler();

	button1 = new TButton(this, CM_OKBUTTON1, "OK", 50, 270, 63, 39);
	new TButton(this, -1, "Nothing", 50, 200, 63, 39);
	button2 = new TODButton(this, CM_OKBUTTON2, "", 150, 270, 63, 39,
		BM_OKNORMAL, BM_OKPRESSED, BM_OKFOCUSED);
}

///////////////////////////////////////////////////////////
// TWndw::~TWndw()
///////////////////////////////////////////////////////////
TWndw::~TWndw()
{
	// Perform object cleanup here.
}

///////////////////////////////////////////////////////////
// TWndw::SetupWindow()
///////////////////////////////////////////////////////////
void TWndw::SetupWindow()
{
	// Call base class's SetupWindow.
	TFrameWindow::SetupWindow();

	// Perform initialization that requires a
	// valid window handle here.
}

///////////////////////////////////////////////////////////
// TWndw::CleanupWindow()
///////////////////////////////////////////////////////////
void TWndw::CleanupWindow()
{
	// Perform cleanup that requires a valid
	// window handle here.
}


//////////////////////////////////
// The application class.
//////////////////////////////////
class TApp : public TApplication
{
public:
	TApp(): TApplication() {}
	void InitMainWindow();
};

///////////////////////////////////////////////////////////
// TApp::InitMainWindow()
///////////////////////////////////////////////////////////
void TApp::InitMainWindow()
{
	TFrameWindow *wndw = new TWndw(0, "Owner-Draw Button in FrameWindow");
	SetMainWindow(wndw);
	EnableCtl3d();
//	EnableBWCC();
}

///////////////////////////////////////////////////////////
// OwlMain()
///////////////////////////////////////////////////////////
int OwlMain(int, char*[])
{
	return TApp().Run();
}

